--[[

$Revision: 32 $

(C) Copyright 2009 Bethink (bethink at naef dot com)
See LICENSE.txt for license terms.

]]


----------------------------------------------------------------------
-- Initialization

BossNotesAbilities = LibStub("AceAddon-3.0"):NewAddon("BossNotesAbilities",
	"AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("BossNotes")

	
----------------------------------------------------------------------
-- Core handlers

-- One-time initialization
function BossNotesAbilities:OnInitialize ()
	-- Use the BossNotes DB. This is OK as long as this module is hard-embedded.
	self.db = BossNotes.db
	
	-- Initialize spell data structure
	if not self.db.global.npcs then
		self.db.global.npcs = { }
	end

	-- Register collection
	BossNotes:RegisterCollection(L["ABILITIES"], BossNotesAbilities_Query)
end

-- Handles the (re-)enabling of this add-on
function BossNotesAbilities:OnEnable ()
	-- Register events
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end



----------------------------------------------------------------------
-- API

-- Returns the abilities note
function BossNotesAbilities_Query (instance, encounter)
	-- Only handle encounters
	if not encounter then
		return nil
	end
	
	-- Build HTML
	local html = { }
	table.insert(html, "<html><body>")
	for _, npcId in ipairs(encounter.npcIds) do
		local npc = BossNotesAbilities.db.global.npcs[npcId]
		if npc then
			-- Header
			table.insert(html, "<h1>" .. npc.name .. "</h1>")
			
			-- Acquire and sort spells
			local spells = { }
			for _, spell in pairs(npc.spells) do
				table.insert(spells, spell)
			end
			table.sort(spells, function (a, b)
				if a.difficulty ~= b.difficulty then
					return a.difficulty < b.difficulty
				end
				return a.name < b.name
			end)
			
			-- Append spells
			for _, spell in ipairs(spells) do
				if spell.difficulty ~= 2 then
					table.insert(html, string.format(L["SPELL"], spell.spellId, spell.name))
				else
					table.insert(html, string.format(L["SPELL_HEROIC"], spell.spellId, spell.name))
				end
			end
			table.insert(html, "<br />")
		end
	end
	
	-- Placeholder
	if #html == 1 then
		table.insert(html, L["ABILITIES_NOT_RECORDED"])
	end
	
	-- Finish
	table.insert(html, "</body></html>")
	
	-- Return
	return table.concat(html, "\n")
end


----------------------------------------------------------------------
-- Events

-- Handles a combat log event
function BossNotesAbilities:COMBAT_LOG_EVENT_UNFILTERED (message, ...)
	-- Check for NPC
	local guid = select(3, ...)
	if string.sub(guid, 5, 5) ~= "3" then
		return
	end
	
	-- Check for boss
	local npcId = tonumber("0x" .. string.sub(guid, 9, 12))
	if not BossNotes:GetInstanceAndEncounter(npcId) then
		return
	end
	
	-- Check event
	local event = select(2, ...)
	if event ~= "SPELL_CAST_START" and event ~= "SPELL_CAST_SUCCESS" then
		return
	end

	-- Track spell
	local npcName = select(4, ...)
	local spellId = select(9, ...)
	local spellName = select(10, ...)
	self:TrackSpell(npcId, npcName, spellId, spellName)
end


----------------------------------------------------------------------
-- Support methods

-- Tracks a spell
function BossNotesAbilities:TrackSpell (npcId, npcName, spellId, spellName, difficulty)
	-- Track NPC
	local npc = self.db.global.npcs[npcId]
	if not npc then
		self.db.global.npcs[npcId] = {
			npcId = npcId,
			name = npcName,
			spells = { }
		}
		npc = self.db.global.npcs[npcId]
	end
	
	-- Track spell
	if not npc.spells[spellId] then
		-- Add spell
		npc.spells[spellId] = {
			spellId = spellId,
			name = spellName,
			difficulty = GetInstanceDifficulty()
		}
		
		-- Invalidate
		local instance, encounter = BossNotes:GetInstanceAndEncounter(npcId)
		local key = BossNotes:GetContextKey(instance, encounter)
		BossNotes:InvalidateContextKey(key)
	end
end
